/*********************************************************************NVMH3****
$Revision: #3 $

Copyright NVIDIA Corporation 2007
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY
LOSS) ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF
NVIDIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

% Similar to the classic "RenderMan Companion" wood shader, though
% for realtime performance we use a noise texture rather than
% calls to a numeric noise() function. This texture can be loaded
% from disk, or dynamically created on the spot by
% DirectX using the HLSL virtual machine.
% This new version is updated to support varying shininess for light
% and dark bands in the wood.

keywords: material organic pattern virtual_machine


To learn more about shading, shaders, and to bounce ideas off other shader
    authors and users, visit the NVIDIA Shader Library Forums at:

    http://developer.nvidia.com/forums/

******************************************************************************/

// set this to use a dynamically-created noise texture
#define PROC_NOISE

/************* UN-TWEAKABLES **************/

//// UN-TWEAKABLES - AUTOMATICALLY-TRACKED TRANSFORMS ////////////////

float4x4 matWorldInv;
float4x4 matWorldViewProj;
float4x4 matWorld;
float4x4 matViewInv;

/************************************************************/
/*** tweakables *********************************************/
/************************************************************/

/// Point Lamp 0 ////////////
float3 Lamp0Pos : Position = {-2000.0f,1000.0f,-2000.0f};
float3 Lamp0Color : Specular = {1.0f,1.0f,1.0f};

////

float3 AmbiColor : Ambient = {0.17f, 0.17f, 0.17f};

////

float3 WoodColor1 = {0.85f, 0.55f, 0.01f};
float3 WoodColor2 = {0.60f, 0.41f, 0.0f};
float Ks1 = 0.6;// 0.5
float Ks2 = 0.7;
float SpecExpon : SpecularPower = 50.0;
float RingScale = 0.46;
float AmpScale = 0.5;// 0.7

// These attributes mean a WOODEN Log, not a Logarithm!
//	They define the relative size of the log to the model.
float WoodScale = 1;

float WOffX = -100.0;

float WOffY = 0.0;

float WOffZ = 100.0;

static float3 WoodOffset = (float3(WOffX,WOffY,WOffZ));

// shared shadow mapping supported in Cg version

/////////////

texture NoiseTex <
    string ResourceName = "noiseL8_32x32x32.dds";
    string UIName =  "Noise";
    string ResourceType = "3D";
>;

sampler3D NoiseSamp = sampler_state
 {
    Texture = <NoiseTex>;
    MagFilter = Point;
    MinFilter = Point;
    MipFilter = Point;
    AddressU = Wrap;
    AddressV = Wrap;
    AddressW = Wrap;
};

/************* DATA STRUCTS **************/

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
    float4 Tangent	: TANGENT0;
    float4 Binormal	: BINORMAL0;
};

/* data passed from vertex shader to pixel shader */
struct woodVertexOutput {
    float4 HPosition	: POSITION;
    float3 WoodPos	: TEXCOORD0; // wood grain coordinate system
    float3 LightVec	: TEXCOORD1;
    float3 WorldNormal	: TEXCOORD2;
    float3 WorldView	: TEXCOORD3;
};

/*********** vertex shader ******/

woodVertexOutput mainVS(appdata IN)
{
    woodVertexOutput OUT = (woodVertexOutput)0;
    OUT.WorldNormal = mul(IN.Normal,matWorldInv).xyz;
    float4 Po = float4(IN.Position.xyz,1);
    float3 Pw = mul(Po,matWorld).xyz;
    OUT.LightVec = Lamp0Pos - Pw;
    OUT.WorldView = (matViewInv[3].xyz - Pw);
    float4 hpos = mul(Po,matWorldViewProj);
    OUT.HPosition = hpos;
    //
    // This shader uses the object coordinates to determine the wood-grain
    //   coordinate system at shader runtime. Alternatively, you could bake
    //	 the coordinate system into the model as an alternative texcoord. The
    //	 current method applies to all possible models, while baking-in lets
    //	 you try different tricks such as modeling the grain of bent wood,
    //	 say for a bow or for the hull timbers of a ship.
    //
    OUT.WoodPos = (WoodScale*Po.xyz) + WoodOffset; // wood grain coordinate system
    return OUT;
}

/********* pixel shader ********/

float4 woodPS(woodVertexOutput IN) : COLOR
{
    float3 Ln = normalize(IN.LightVec);
    float3 Nn = normalize(IN.WorldNormal);
    float3 Pwood = IN.WoodPos + (AmpScale * tex3D(NoiseSamp,IN.WoodPos.xyz/32.0).xyz);
    float r = RingScale * sqrt(dot(Pwood.yz,Pwood.yz));
    r = r + tex3D(NoiseSamp,r.xxx/32.0).x;
    r = r - floor(r);
    r = smoothstep(0.0, 0.8, r) - smoothstep(0.83,1.0,r);
    float3 dColor = lerp(WoodColor1,WoodColor2,r);
    float Ks = lerp(Ks1,Ks2,r);
    float3 Vn = normalize(IN.WorldView);
    float3 Hn = normalize(Vn + Ln);
    float hdn = dot(Hn,Nn);
    float ldn = dot(Ln,Nn);
    float4 litV = lit(ldn,hdn,SpecExpon);
    float3 diffContrib = dColor * ((litV.y*Lamp0Color) + AmbiColor);
    float3 specContrib = Ks * litV.z * Lamp0Color;
    float3 result = diffContrib + specContrib;
    return float4(result,1);
}

/*************/

technique Main
{
    pass p0
    {		
	VertexShader = compile vs_2_0 mainVS();
	    ZEnable = true;
		ZWriteEnable = true;
		ZFunc = LessEqual;
		AlphaBlendEnable = false;
		CullMode = None;
	PixelShader = compile ps_2_0 woodPS();
    }
}

/***************************** eof ***/
